import logging

from aiohttp import web
from aiohttp_apispec import docs, match_info_schema, request_schema
from dexa_sdk.managers.ada_manager import V2ADAManager
from mydata_did.v1_0.routes.maps.tag_maps import TAGS_JSONLD_FUNCTIONS_LABEL
from mydata_did.v1_0.routes.openapi.schemas import (
    SendJSONLDDIDCommProcessedDataMessageHandlerMatchInfoSchema,
    SendJSONLDDIDCommProcessedDataMessageHandlerRequestSchema,
)

LOGGER = logging.getLogger(__name__)

PAGINATION_PAGE_SIZE = 10


@docs(
    tags=[TAGS_JSONLD_FUNCTIONS_LABEL],
    summary="Send JSON-LD processed-data didcomm message to the remote agent.",
    responses={
        204: {
            "description": "Success",
        },
    },
)
@match_info_schema(SendJSONLDDIDCommProcessedDataMessageHandlerMatchInfoSchema())
@request_schema(SendJSONLDDIDCommProcessedDataMessageHandlerRequestSchema())
async def send_json_ld_didcomm_processed_data_message_handler(request: web.BaseRequest):
    """Send JSON-LD didcomm processed data message handler."""

    # Context.
    context = request.app["request_context"]

    # Fetch path parameters.
    connection_id = request.match_info["connection_id"]

    # Fetch request body
    body = await request.json()
    data = body.get("data")
    signature_options = body.get("signature_options")

    # Initialise MyData DID Manager.
    mgr = V2ADAManager(context=context)

    await mgr.send_json_ld_processed_message(
        connection_id=connection_id,
        data=data,
        signature_options=signature_options,
    )

    return web.json_response({}, status=204)



from aries_cloudagent.messaging.agent_message import AgentMessage, AgentMessageSchema
from marshmallow import EXCLUDE, fields
from mydata_did.v1_0.message_types import DA_PERMISSIONS, PROTOCOL_PACKAGE
from mydata_did.v1_0.models.da_permissions_model import (
    DAPermissionsBodyModel,
    DAPermissionsBodyModelSchema,
)

HANDLER_CLASS = (
    f"{PROTOCOL_PACKAGE}.handlers.da_permissions_handler.DAPermissionsMessageHandler"
)


class DAPermissionsMessage(AgentMessage):
    class Meta:
        handler_class = HANDLER_CLASS
        message_type = DA_PERMISSIONS
        schema_class = "DAPermissionsMessageSchema"

    def __init__(self, *, body: DAPermissionsBodyModel, **kwargs):
        super().__init__(**kwargs)

        self.body = body


class DAPermissionsMessageSchema(AgentMessageSchema):
    class Meta:
        model_class = DAPermissionsMessage
        unknown = EXCLUDE

    body = fields.Nested(DAPermissionsBodyModelSchema)



from aries_cloudagent.messaging.models.base import BaseModel, BaseModelSchema
from marshmallow import fields


class UpdatePreferencesBodyModel(BaseModel):
    class Meta:
        schema_class = "UpdatePreferencesBodyModelSchema"

    def __init__(
        self,
        *,
        dda_instance_id: str = None,
        da_instance_id: str = None,
        state: str = None,
        **kwargs
    ):

        super().__init__(**kwargs)

        # Set attributes
        self.dda_instance_id = dda_instance_id
        self.da_instance_id = da_instance_id
        self.state = state


class UpdatePreferencesBodyModelSchema(BaseModelSchema):
    class Meta:
        model_class = UpdatePreferencesBodyModel

    dda_instance_id = fields.Str()
    da_instance_id = fields.Str()
    state = fields.Str()



import json

from aries_cloudagent.messaging.base_handler import (
    BaseHandler,
    BaseResponder,
    RequestContext,
)
from mydata_did.v1_0.messages.data_agreement_qr_code_problem_report import (
    DataAgreementQrCodeProblemReport,
)


class DataAgreementQrCodeProblemReportHandler(BaseHandler):
    """Handle for data-agreement-qr-code/1.0/problem-report message"""

    async def handle(self, context: RequestContext, responder: BaseResponder):
        """
        Message handler logic for data-agreement-qr-code/1.0/problem-report message.
        """

        # Assert if received message is of type DataAgreementQrCodeProblemReport
        assert isinstance(context.message, DataAgreementQrCodeProblemReport)

        self._logger.info(
            "Received data-agreement-qr-code/1.0/problem-report message: \n%s",
            json.dumps(context.message.serialize(), indent=4),
        )



import typing

from aries_cloudagent.messaging.models.base import BaseModel, BaseModelSchema
from marshmallow import EXCLUDE, fields


class FPRControllerDetailsModel(BaseModel):
    class Meta:
        schema_class = "FPRControllerDetailsModelSchema"

    def __init__(
        self,
        organisation_did: str = None,
        organisation_name: str = None,
        cover_image_url: str = None,
        logo_image_url: str = None,
        location: str = None,
        organisation_type: str = None,
        description: str = None,
        policy_url: str = None,
        eula_url: str = None,
        **kwargs
    ):
        super().__init__(**kwargs)

        self.organisation_did = organisation_did
        self.organisation_name = organisation_name
        self.cover_image_url = cover_image_url
        self.logo_image_url = logo_image_url
        self.location = location
        self.organisation_type = organisation_type
        self.description = description
        self.policy_url = policy_url
        self.eula_url = eula_url


class FPRControllerDetailsModelSchema(BaseModelSchema):
    class Meta:
        model_class = FPRControllerDetailsModel
        unknown = EXCLUDE

    organisation_did = fields.Str(required=False)
    organisation_name = fields.Str(required=False)
    cover_image_url = fields.Str(required=False)
    logo_image_url = fields.Str(required=False)
    location = fields.Str(required=False)
    organisation_type = fields.Str(required=False)
    description = fields.Str(required=False)
    policy_url = fields.Str(required=False)
    eula_url = fields.Str(required=False)


class FPRDUSModel(BaseModel):
    class Meta:
        schema_class = "FPRDUSModelSchema"

    def __init__(
        self,
        controller_details: FPRControllerDetailsModel = None,
        dda_instance_id: str = None,
        dda_instance_permission_state: str = None,
        **kwargs
    ):
        super().__init__(**kwargs)

        self.controller_details = controller_details
        self.dda_instance_id = dda_instance_id
        self.dda_instance_permission_state = dda_instance_permission_state


class FPRDUSModelSchema(BaseModelSchema):
    class Meta:
        model_class = FPRDUSModel
        unknown = EXCLUDE

    controller_details = fields.Nested(FPRControllerDetailsModelSchema, required=False)
    dda_instance_id = fields.Str(required=False)
    dda_instance_permission_state = fields.Str(required=False)


class FPRPrefsModel(BaseModel):
    class Meta:
        schema_class = "FPRPrefsModelSchema"

    def __init__(
        self,
        instance_id: str = None,
        instance_permission_state: str = None,
        dus: typing.List[FPRDUSModel] = None,
        sector: str = None,
        **kwargs
    ):
        super().__init__(**kwargs)

        # Set attributes
        self.instance_id = instance_id
        self.instance_permission_state = instance_permission_state
        self.dus = dus
        self.sector = sector


class FPRPrefsModelSchema(BaseModelSchema):
    class Meta:
        model_class = FPRPrefsModel

    instance_id = fields.Str(required=False)
    instance_permission_state = fields.Str(required=False)
    sector = fields.Str(required=False)
    dus = fields.List(fields.Nested(FPRDUSModelSchema), required=False)


class FetchPreferencesResponseBody(BaseModel):
    class Meta:
        schema_class = "FetchPreferencesResponseBodySchema"

    def __init__(
        self,
        *,
        prefs: typing.List[FPRPrefsModel] = None,
        sectors: typing.List[str] = None,
        **kwargs
    ):
        super().__init__(**kwargs)

        # Set attributes
        self.prefs = prefs
        self.sectors = sectors


class FetchPreferencesResponseBodySchema(BaseModelSchema):
    class Meta:
        model_class = FetchPreferencesResponseBody

    prefs = fields.List(fields.Nested(FPRPrefsModelSchema), required=False)
    sectors = fields.List(fields.Str, required=False)



import json

from aries_cloudagent.messaging.base_handler import (
    BaseHandler,
    BaseResponder,
    RequestContext,
)
from dexa_sdk.managers.ada_manager import V2ADAManager
from mydata_did.v1_0.messages.json_ld_processed import JSONLDProcessedMessage


class JSONLDProcessedHandler(BaseHandler):
    """Handle for JSONLD processed message"""

    async def handle(self, context: RequestContext, responder: BaseResponder):
        """
        Message handler logic.
        """

        assert isinstance(context.message, JSONLDProcessedMessage)

        self._logger.info(
            "Received JSONLD processed message: \n%s",
            json.dumps(context.message.serialize(), indent=4),
        )

        # Initialize ADA manager
        mgr = V2ADAManager(context)

        # Call the function

        await mgr.process_json_ld_processed_message(
            json_ld_processed_message=context.message,
            receipt=context.message_receipt,
        )



import json

from aries_cloudagent.messaging.base_handler import (
    BaseHandler,
    BaseResponder,
    RequestContext,
)
from mydata_did.v1_0.messages.json_ld_processed_response import (
    JSONLDProcessedResponseMessage,
)


class JSONLDProcessedResponseHandler(BaseHandler):
    """Handle for JSONLD processed response message"""

    async def handle(self, context: RequestContext, responder: BaseResponder):
        """
        Message handler logic for JSONLD processed response-data-response message.
        """

        # Assert if received message is of type JSONLDProcessedResponseMessage
        assert isinstance(context.message, JSONLDProcessedResponseMessage)

        self._logger.info(
            "Received JSONLD processed response-data-response message: \n%s",
            json.dumps(context.message.serialize(), indent=4),
        )



from aries_cloudagent.messaging.agent_message import AgentMessage, AgentMessageSchema
from marshmallow import EXCLUDE, fields
from mydata_did.v1_0.message_types import (
    DATA_AGREEMENT_NEGOTIATION_REJECT,
    PROTOCOL_PACKAGE,
)
from mydata_did.v1_0.models.data_agreement_negotiation_reject_model import (
    DataAgreementNegotiationRejectBody,
    DataAgreementNegotiationRejectBodySchema,
)

# Handler class for data agreement reject message
HANDLER_CLASS = (
    f"{PROTOCOL_PACKAGE}.handlers"
    ".data_agreement_negotiation_reject_handler.DataAgreementNegotiationRejectMessageHandler"
)


class DataAgreementNegotiationRejectMessage(AgentMessage):
    """
    Message class for data agreement negotiation reject message.
    """

    class Meta:
        # Handler class that can handle this message
        handler_class = HANDLER_CLASS

        # Message type
        message_type = DATA_AGREEMENT_NEGOTIATION_REJECT

        # Message schema class
        schema_class = "DataAgreementNegotiationRejectMessageSchema"

    def __init__(self, *, body: DataAgreementNegotiationRejectBody, **kwargs):
        """
        Initialize a DataAgreementNegotiationAcceptMessage message instance.
        """
        super().__init__(**kwargs)

        # Message body
        self.body = body


class DataAgreementNegotiationRejectMessageSchema(AgentMessageSchema):
    """
    Schema class for data agreement negotiation reject message
    """

    class Meta:
        # The message class that this schema is for
        model_class = DataAgreementNegotiationRejectMessage

        # Unknown fields to exclude from the schema (handled by marshmallow)
        unknown = EXCLUDE

    # Message body
    body = fields.Nested(DataAgreementNegotiationRejectBodySchema, required=True)



from aries_cloudagent.messaging.agent_message import AgentMessage, AgentMessageSchema
from marshmallow import EXCLUDE, fields
from mydata_did.v1_0.message_types import (
    PROTOCOL_PACKAGE,
    THIRDPARTY_DATA_SHARING_UPDATE_PREFERENCES,
)
from mydata_did.v1_0.models.update_preferences_model import (
    UpdatePreferencesBodyModel,
    UpdatePreferencesBodyModelSchema,
)

HANDLER_CLASS = f"{PROTOCOL_PACKAGE}.handlers.update_preferences_message_handler.UpdatePreferencesMessageHandler"


class UpdatePreferencesMessage(AgentMessage):
    class Meta:
        handler_class = HANDLER_CLASS
        message_type = THIRDPARTY_DATA_SHARING_UPDATE_PREFERENCES
        schema_class = "UpdatePreferencesMessageSchema"

    def __init__(self, *, body: UpdatePreferencesBodyModel, **kwargs):
        super().__init__(**kwargs)

        self.body = body


class UpdatePreferencesMessageSchema(AgentMessageSchema):
    class Meta:
        model_class = UpdatePreferencesMessage
        unknown = EXCLUDE

    body = fields.Nested(UpdatePreferencesBodyModelSchema)



from aries_cloudagent.messaging.models.base import BaseModel, BaseModelSchema
from marshmallow import EXCLUDE, fields


class DataAgreementResponseBody(BaseModel):
    """
    Data agreement response body model class
    """

    class Meta:
        schema_class = "DataAgreementResponseBodySchema"

    def __init__(self, *, data_agreement_id: str, **kwargs):
        super().__init__(**kwargs)
        self.data_agreement_id = data_agreement_id


class DataAgreementResponseBodySchema(BaseModelSchema):
    """
    Data agreement response schema class
    """

    class Meta:
        model_class = DataAgreementResponseBody
        unknown = EXCLUDE

    data_agreement_id = fields.Str(
        data_key="data_agreement_id",
        example="45988dd2-62b9-4ede-8189-fb99c64b42d1",
        description="Data agreement identifier",
    )
